@@ -14,7 +14,7 @@ from datetime import datetime, timedelta
 from functools import partial
 from http import HTTPStatus, cookies
 from http.cookies import Morsel, SimpleCookie
-from ipaddress import IPv4Address, IPv6Address
+from ipaddress import IPv4Address, IPv6Address, ip_address
 from pathlib import Path
 from typing import Any, Literal, cast
 from urllib.parse import SplitResult
@@ -54,6 +54,7 @@ from .data import (
     SmartDetectTrack,
     Version,
     Viewer,
+    WSAction,
     WSPacket,
     WSSubscriptionMessage,
     create_from_unifi_dict,
@@ -64,6 +65,7 @@ from .data.types import IteratorCallback, ProgressCallback
 from .exceptions import BadRequest, NotAuthorized, NvrError
 from .utils import (
     decode_token_cookie,
+    format_host_for_url,
     get_response_reason,
     ip_from_host,
     pybool_to_json_bool,
@@ -217,10 +219,14 @@ class BaseApiClient:
 
     headers: dict[str, str] | None = None
     _private_websocket: Websocket | None = None
+    _events_websocket: Websocket | None = None
+    _devices_websocket: Websocket | None = None
 
     private_api_path: str = "/proxy/protect/api/"
     public_api_path: str = "/proxy/protect/integration"
     private_ws_path: str = "/proxy/protect/ws/updates"
+    events_ws_path: str = "/proxy/protect/integration/v1/subscribe/events"
+    devices_ws_path: str = "/proxy/protect/integration/v1/subscribe/devices"
 
     cache_dir: Path
     config_dir: Path
@@ -273,12 +279,26 @@ class BaseApiClient:
 
     def _update_url(self) -> None:
         """Updates the url after changing _host or _port."""
+        formatted_host = format_host_for_url(self._host)
+
         if self._port != 443:
-            self._url = URL(f"https://{self._host}:{self._port}")
-            self._ws_url = URL(f"wss://{self._host}:{self._port}{self.private_ws_path}")
+            self._url = URL(f"https://{formatted_host}:{self._port}")
+            self._ws_url = URL(
+                f"wss://{formatted_host}:{self._port}{self.private_ws_path}"
+            )
+            self._events_ws_url = URL(
+                f"https://{formatted_host}:{self._port}{self.events_ws_path}"
+            )
+            self._devices_ws_url = URL(
+                f"https://{formatted_host}:{self._port}{self.devices_ws_path}"
+            )
         else:
-            self._url = URL(f"https://{self._host}")
-            self._ws_url = URL(f"wss://{self._host}{self.private_ws_path}")
+            self._url = URL(f"https://{formatted_host}")
+            self._ws_url = URL(f"wss://{formatted_host}{self.private_ws_path}")
+            self._events_ws_url = URL(f"https://{formatted_host}{self.events_ws_path}")
+            self._devices_ws_url = URL(
+                f"https://{formatted_host}{self.devices_ws_path}"
+            )
 
         self.base_url = str(self._url)
 
@@ -294,6 +314,16 @@ class BaseApiClient:
         """Get Websocket URL."""
         return str(self._ws_url_object)
 
+    @property
+    def events_ws_url(self) -> str:
+        """Get Events Websocket URL."""
+        return str(self._events_ws_url)
+
+    @property
+    def devices_ws_url(self) -> str:
+        """Get Devices Websocket URL."""
+        return str(self._devices_ws_url)
+
     @property
     def config_file(self) -> Path:
         return self.config_dir / "unifi_protect.json"
@@ -329,6 +359,15 @@ class BaseApiClient:
         await self.ensure_authenticated()
         return self.headers
 
+    async def _auth_public_api_websocket(
+        self, force: bool = False
+    ) -> dict[str, str] | None:
+        """Authenticate for Public API Websocket."""
+        if self._api_key is None:
+            raise NotAuthorized("API key is required for public API WebSocket")
+
+        return {"X-API-KEY": self._api_key}
+
     def _get_websocket(self) -> Websocket:
         """Gets or creates current Websocket."""
         if self._private_websocket is None:
@@ -345,6 +384,38 @@ class BaseApiClient:
             )
         return self._private_websocket
 
+    def _get_events_websocket(self) -> Websocket:
+        """Gets or creates current Events Websocket."""
+        if self._events_websocket is None:
+            self._events_websocket = Websocket(
+                lambda: self._events_ws_url,
+                self._auth_public_api_websocket,
+                lambda: None,
+                self.get_public_api_session,
+                self._process_events_ws_message,
+                self._on_events_websocket_state_change,
+                verify=self._verify_ssl,
+                timeout=self._ws_timeout,
+                receive_timeout=self._ws_receive_timeout,
+            )
+        return self._events_websocket
+
+    def _get_devices_websocket(self) -> Websocket:
+        """Gets or creates current Devices Websocket."""
+        if self._devices_websocket is None:
+            self._devices_websocket = Websocket(
+                lambda: self._devices_ws_url,
+                self._auth_public_api_websocket,
+                lambda: None,
+                self.get_public_api_session,
+                self._process_devices_ws_message,
+                self._on_devices_websocket_state_change,
+                verify=self._verify_ssl,
+                timeout=self._ws_timeout,
+                receive_timeout=self._ws_receive_timeout,
+            )
+        return self._devices_websocket
+
     def _update_bootstrap_soon(self) -> None:
         """Update bootstrap soon."""
         _LOGGER.debug("Updating bootstrap soon")
@@ -636,8 +707,25 @@ class BaseApiClient:
             response = await self.request("post", url=url, json=auth)
             if response.status != 200:
                 await self._raise_for_status(response, True)
-            self.set_header("cookie", response.headers.get("set-cookie", ""))
+
+            csrf_token = response.headers.get("x-csrf-token")
+            if csrf_token:
+                self.set_header("x-csrf-token", csrf_token)
+
+            cookie_header = response.headers.get("set-cookie", "")
+            self.set_header("cookie", cookie_header)
             self._is_authenticated = True
+
+            # parse and store the cookie for session persistence
+            if self.store_sessions and cookie_header:
+                # extract cookie from header to save in session file
+                cookie = SimpleCookie(cookie_header)
+                if cookie:
+                    for cookie_obj in cookie.values():
+                        self._last_token_cookie = cookie_obj
+                        await self._update_auth_config(cookie_obj)
+                        break  # auth response only contains single cookie (TOKEN or UOS_TOKEN)
+
             _LOGGER.debug("Authenticated successfully!")
 
     async def _update_last_token_cookie(self, response: aiohttp.ClientResponse) -> None:
@@ -649,7 +737,6 @@ class BaseApiClient:
             and csrf_token != self.headers.get("x-csrf-token")
         ):
             self.set_header("x-csrf-token", csrf_token)
-            await self._update_last_token_cookie(response)
             self._update_cookiename(response.cookies)
 
         if (
@@ -736,10 +823,16 @@ class BaseApiClient:
         cookie_value = _COOKIE_RE.sub("", str(cookie[cookie_name]))
         self._last_token_cookie = cookie[cookie_name]
         self._last_token_cookie_decode = None
+
+        # Only mark as authenticated if we have both cookie and CSRF token
+        csrf_token = session.get("csrf")
+        if not csrf_token:
+            _LOGGER.debug("Session found but missing CSRF token, will re-authenticate")
+            return None
+
         self._is_authenticated = True
         self.set_header("cookie", cookie_value)
-        if session.get("csrf"):
-            self.set_header("x-csrf-token", session["csrf"])
+        self.set_header("x-csrf-token", csrf_token)
         return cookie
 
     def is_authenticated(self) -> bool:
@@ -781,10 +874,28 @@ class BaseApiClient:
             websocket.stop()
             await websocket.wait_closed()
             self._private_websocket = None
+        if self._events_websocket:
+            events_websocket = self._get_events_websocket()
+            events_websocket.stop()
+            await events_websocket.wait_closed()
+            self._events_websocket = None
+        if self._devices_websocket:
+            devices_websocket = self._get_devices_websocket()
+            devices_websocket.stop()
+            await devices_websocket.wait_closed()
+            self._devices_websocket = None
 
     def _process_ws_message(self, msg: aiohttp.WSMessage) -> None:
         raise NotImplementedError
 
+    def _process_events_ws_message(self, msg: aiohttp.WSMessage) -> None:
+        """Process events websocket message - to be implemented by subclass."""
+        raise NotImplementedError
+
+    def _process_devices_ws_message(self, msg: aiohttp.WSMessage) -> None:
+        """Process devices websocket message - to be implemented by subclass."""
+        raise NotImplementedError
+
     def _get_last_update_id(self) -> str | None:
         raise NotImplementedError
 
@@ -795,6 +906,14 @@ class BaseApiClient:
         """Websocket state changed."""
         _LOGGER.debug("Websocket state changed: %s", state)
 
+    def _on_events_websocket_state_change(self, state: WebsocketState) -> None:
+        """Events websocket state changed."""
+        _LOGGER.debug("Events websocket state changed: %s", state)
+
+    def _on_devices_websocket_state_change(self, state: WebsocketState) -> None:
+        """Devices websocket state changed."""
+        _LOGGER.debug("Devices websocket state changed: %s", state)
+
 
 class ProtectApiClient(BaseApiClient):
     """
@@ -832,7 +951,11 @@ class ProtectApiClient(BaseApiClient):
     _subscribed_models: set[ModelType]
     _ignore_stats: bool
     _ws_subscriptions: list[Callable[[WSSubscriptionMessage], None]]
+    _events_ws_subscriptions: list[Callable[[WSSubscriptionMessage], None]]
+    _devices_ws_subscriptions: list[Callable[[WSSubscriptionMessage], None]]
     _ws_state_subscriptions: list[Callable[[WebsocketState], None]]
+    _events_ws_state_subscriptions: list[Callable[[WebsocketState], None]]
+    _devices_ws_state_subscriptions: list[Callable[[WebsocketState], None]]
     _bootstrap: Bootstrap | None = None
     _last_update_dt: datetime | None = None
     _connection_host: IPv4Address | IPv6Address | str | None = None
@@ -881,16 +1004,62 @@ class ProtectApiClient(BaseApiClient):
         self._subscribed_models = subscribed_models or set()
         self._ignore_stats = ignore_stats
         self._ws_subscriptions = []
+        self._events_ws_subscriptions = []
+        self._devices_ws_subscriptions = []
         self._ws_state_subscriptions = []
+        self._events_ws_state_subscriptions = []
+        self._devices_ws_state_subscriptions = []
         self.ignore_unadopted = ignore_unadopted
         self._update_lock = asyncio.Lock()
 
         if override_connection_host:
-            self._connection_host = ip_from_host(self._host)
+            self._connection_host = ip_address(self._host)
 
         if debug:
             set_debug()
 
+    def _set_connection_host_from_bootstrap(self) -> None:
+        """
+        Set connection host from bootstrap NVR hosts (sync).
+
+        NOTE: Must stay in sync with _async_set_connection_host_from_bootstrap().
+        Sync version for property getter, async version for update() method.
+        """
+        index = 0
+        try:
+            index = self.bootstrap.nvr.hosts.index(self._host)
+        except ValueError:
+            try:
+                host = ip_address(self._host)
+                with contextlib.suppress(ValueError):
+                    index = self.bootstrap.nvr.hosts.index(host)
+            except ValueError:
+                pass
+
+        self._connection_host = self.bootstrap.nvr.hosts[index]
+
+    async def _async_set_connection_host_from_bootstrap(
+        self,
+        bootstrap: Bootstrap,
+    ) -> None:
+        """
+        Set connection host from bootstrap NVR hosts (async).
+
+        NOTE: Must stay in sync with _set_connection_host_from_bootstrap().
+        Async version allows DNS resolution via ip_from_host().
+        """
+        index = 0
+        try:
+            index = bootstrap.nvr.hosts.index(self._host)
+        except ValueError:
+            try:
+                host_ip = await ip_from_host(self._host)
+                index = bootstrap.nvr.hosts.index(host_ip)
+            except ValueError:
+                pass
+
+        self._connection_host = bootstrap.nvr.hosts[index]
+
     @cached_property
     def bootstrap(self) -> Bootstrap:
         if self._bootstrap is None:
@@ -900,21 +1069,11 @@ class ProtectApiClient(BaseApiClient):
 
     @property
     def connection_host(self) -> IPv4Address | IPv6Address | str:
-        """Connection host to use for generating RTSP URLs"""
+        """Connection host to use for generating RTSP URLs."""
         if self._connection_host is None:
-            # fallback if cannot find user supplied host
-            index = 0
-            try:
-                # check if user supplied host is avaiable
-                index = self.bootstrap.nvr.hosts.index(self._host)
-            except ValueError:
-                # check if IP of user supplied host is avaiable
-                host = ip_from_host(self._host)
-                with contextlib.suppress(ValueError):
-                    index = self.bootstrap.nvr.hosts.index(host)
-
-            self._connection_host = self.bootstrap.nvr.hosts[index]
+            self._set_connection_host_from_bootstrap()
 
+        assert self._connection_host is not None
         return self._connection_host
 
     async def update(self) -> Bootstrap:
@@ -948,6 +1107,11 @@ class ProtectApiClient(BaseApiClient):
                 )
             self.__dict__.pop("bootstrap", None)
             self._bootstrap = bootstrap
+
+            # Set connection host if not set via override_connection_host
+            if self._connection_host is None:
+                await self._async_set_connection_host_from_bootstrap(bootstrap)
+
             return bootstrap
 
     async def poll_events(self) -> None:
@@ -989,6 +1153,62 @@ class ProtectApiClient(BaseApiClient):
             except Exception:
                 _LOGGER.exception("Exception while running subscription handler")
 
+    def emit_events_message(self, msg: WSSubscriptionMessage) -> None:
+        """Emit message to all events subscriptions."""
+        if _LOGGER.isEnabledFor(logging.DEBUG):
+            if msg.new_obj is not None:
+                _LOGGER.debug(
+                    "emitting events message: %s:%s:%s:%s",
+                    msg.action,
+                    msg.new_obj.model,
+                    msg.new_obj.id,
+                    list(msg.changed_data),
+                )
+            elif msg.old_obj is not None:
+                _LOGGER.debug(
+                    "emitting events message: %s:%s:%s",
+                    msg.action,
+                    msg.old_obj.model,
+                    msg.old_obj.id,
+                )
+            else:
+                _LOGGER.debug("emitting events message: %s", msg.action)
+
+        for sub in self._events_ws_subscriptions:
+            try:
+                sub(msg)
+            except Exception:
+                _LOGGER.exception("Exception while running events subscription handler")
+
+    def emit_devices_message(self, msg: WSSubscriptionMessage) -> None:
+        """Emit message to all devices subscriptions."""
+        if _LOGGER.isEnabledFor(logging.DEBUG):
+            if msg.new_obj is not None:
+                _LOGGER.debug(
+                    "emitting devices message: %s:%s:%s:%s",
+                    msg.action,
+                    msg.new_obj.model,
+                    msg.new_obj.id,
+                    list(msg.changed_data),
+                )
+            elif msg.old_obj is not None:
+                _LOGGER.debug(
+                    "emitting devices message: %s:%s:%s",
+                    msg.action,
+                    msg.old_obj.model,
+                    msg.old_obj.id,
+                )
+            else:
+                _LOGGER.debug("emitting devices message: %s", msg.action)
+
+        for sub in self._devices_ws_subscriptions:
+            try:
+                sub(msg)
+            except Exception:
+                _LOGGER.exception(
+                    "Exception while running devices subscription handler"
+                )
+
     def _get_last_update_id(self) -> str | None:
         if self._bootstrap is None:
             return None
@@ -1006,7 +1226,111 @@ class ProtectApiClient(BaseApiClient):
 
         self.emit_message(processed_message)
 
-    async def _get_event_paginate(
+    def _process_events_ws_message(self, msg: aiohttp.WSMessage) -> None:
+        """Process events websocket message (Public API - JSON format)."""
+        if msg.type != aiohttp.WSMsgType.TEXT:
+            _LOGGER.debug("Ignoring non-text websocket message: %s", msg.type)
+            return
+
+        try:
+            data = orjson.loads(msg.data)
+            action_type = data.get("type")  # "update", "add", "remove"
+            item = data.get("item", {})
+            model_key = item.get("modelKey")
+
+            if not action_type or not model_key:
+                _LOGGER.debug("Invalid public API websocket message: %s", data)
+                return
+
+            # Create a WSSubscriptionMessage similar to private WS
+            model_type = ModelType.from_string(model_key)
+
+            if model_type is ModelType.UNKNOWN:
+                _LOGGER.debug("Unknown model type in public API message: %s", model_key)
+                return
+
+            # Create proper objects from the data
+            new_obj: ProtectModelWithId | None = None
+            old_obj: ProtectModelWithId | None = None
+            update_id = item.get("id", "")
+
+            if action_type in ("add", "update"):
+                try:
+                    new_obj = cast(
+                        ProtectModelWithId, create_from_unifi_dict(item, api=self)
+                    )
+                except Exception:
+                    _LOGGER.debug(
+                        "Could not create object from public API data: %s", item
+                    )
+
+            msg_obj = WSSubscriptionMessage(
+                action=WSAction(action_type),
+                new_update_id=update_id,
+                changed_data=item,
+                new_obj=new_obj,
+                old_obj=old_obj,
+            )
+
+            self.emit_events_message(msg_obj)
+        except Exception as e:
+            _LOGGER.exception(
+                "Error processing public API events websocket message: %s", e
+            )
+
+    def _process_devices_ws_message(self, msg: aiohttp.WSMessage) -> None:
+        """Process devices websocket message (Public API - JSON format)."""
+        if msg.type != aiohttp.WSMsgType.TEXT:
+            _LOGGER.debug("Ignoring non-text websocket message: %s", msg.type)
+            return
+
+        try:
+            data = orjson.loads(msg.data)
+            action_type = data.get("type")  # "update", "add", "remove"
+            item = data.get("item", {})
+            model_key = item.get("modelKey")
+
+            if not action_type or not model_key:
+                _LOGGER.debug("Invalid public API websocket message: %s", data)
+                return
+
+            # Create a WSSubscriptionMessage similar to private WS
+            model_type = ModelType.from_string(model_key)
+
+            if model_type is ModelType.UNKNOWN:
+                _LOGGER.debug("Unknown model type in public API message: %s", model_key)
+                return
+
+            # Create proper objects from the data
+            new_obj: ProtectModelWithId | None = None
+            old_obj: ProtectModelWithId | None = None
+            update_id = item.get("id", "")
+
+            if action_type in ("add", "update"):
+                try:
+                    new_obj = cast(
+                        ProtectModelWithId, create_from_unifi_dict(item, api=self)
+                    )
+                except Exception:
+                    _LOGGER.debug(
+                        "Could not create object from public API data: %s", item
+                    )
+
+            msg_obj = WSSubscriptionMessage(
+                action=WSAction(action_type),
+                new_update_id=update_id,
+                changed_data=item,
+                new_obj=new_obj,
+                old_obj=old_obj,
+            )
+
+            self.emit_devices_message(msg_obj)
+        except Exception as e:
+            _LOGGER.exception(
+                "Error processing public API devices websocket message: %s", e
+            )
+
+    async def _get_event_paginate(  # noqa: PLR0912
         self,
         params: dict[str, Any],
         *,
@@ -1064,7 +1388,7 @@ class ProtectApiClient(BaseApiClient):
 
         return events
 
-    async def get_events_raw(
+    async def get_events_raw(  # noqa: PLR0912
         self,
         *,
         start: datetime | None = None,
@@ -1251,6 +1575,34 @@ class ProtectApiClient(BaseApiClient):
         self._get_websocket().start()
         return partial(self._unsubscribe_websocket, ws_callback)
 
+    def subscribe_events_websocket(
+        self,
+        ws_callback: Callable[[WSSubscriptionMessage], None],
+    ) -> Callable[[], None]:
+        """
+        Subscribe to events websocket events.
+
+        Returns a callback that will unsubscribe.
+        """
+        _LOGGER.debug("Adding events subscription: %s", ws_callback)
+        self._events_ws_subscriptions.append(ws_callback)
+        self._get_events_websocket().start()
+        return partial(self._unsubscribe_events_websocket, ws_callback)
+
+    def subscribe_devices_websocket(
+        self,
+        ws_callback: Callable[[WSSubscriptionMessage], None],
+    ) -> Callable[[], None]:
+        """
+        Subscribe to devices websocket events.
+
+        Returns a callback that will unsubscribe.
+        """
+        _LOGGER.debug("Adding devices subscription: %s", ws_callback)
+        self._devices_ws_subscriptions.append(ws_callback)
+        self._get_devices_websocket().start()
+        return partial(self._unsubscribe_devices_websocket, ws_callback)
+
     def _unsubscribe_websocket(
         self,
         ws_callback: Callable[[WSSubscriptionMessage], None],
@@ -1261,6 +1613,26 @@ class ProtectApiClient(BaseApiClient):
         if not self._ws_subscriptions:
             self._get_websocket().stop()
 
+    def _unsubscribe_events_websocket(
+        self,
+        ws_callback: Callable[[WSSubscriptionMessage], None],
+    ) -> None:
+        """Unsubscribe to events websocket events."""
+        _LOGGER.debug("Removing events subscription: %s", ws_callback)
+        self._events_ws_subscriptions.remove(ws_callback)
+        if not self._events_ws_subscriptions:
+            self._get_events_websocket().stop()
+
+    def _unsubscribe_devices_websocket(
+        self,
+        ws_callback: Callable[[WSSubscriptionMessage], None],
+    ) -> None:
+        """Unsubscribe to devices websocket events."""
+        _LOGGER.debug("Removing devices subscription: %s", ws_callback)
+        self._devices_ws_subscriptions.remove(ws_callback)
+        if not self._devices_ws_subscriptions:
+            self._get_devices_websocket().stop()
+
     def subscribe_websocket_state(
         self,
         ws_callback: Callable[[WebsocketState], None],
@@ -1273,6 +1645,30 @@ class ProtectApiClient(BaseApiClient):
         self._ws_state_subscriptions.append(ws_callback)
         return partial(self._unsubscribe_websocket_state, ws_callback)
 
+    def subscribe_events_websocket_state(
+        self,
+        ws_callback: Callable[[WebsocketState], None],
+    ) -> Callable[[], None]:
+        """
+        Subscribe to events websocket state changes.
+
+        Returns a callback that will unsubscribe.
+        """
+        self._events_ws_state_subscriptions.append(ws_callback)
+        return partial(self._unsubscribe_events_websocket_state, ws_callback)
+
+    def subscribe_devices_websocket_state(
+        self,
+        ws_callback: Callable[[WebsocketState], None],
+    ) -> Callable[[], None]:
+        """
+        Subscribe to devices websocket state changes.
+
+        Returns a callback that will unsubscribe.
+        """
+        self._devices_ws_state_subscriptions.append(ws_callback)
+        return partial(self._unsubscribe_devices_websocket_state, ws_callback)
+
     def _unsubscribe_websocket_state(
         self,
         ws_callback: Callable[[WebsocketState], None],
@@ -1280,6 +1676,20 @@ class ProtectApiClient(BaseApiClient):
         """Unsubscribe to websocket state changes."""
         self._ws_state_subscriptions.remove(ws_callback)
 
+    def _unsubscribe_events_websocket_state(
+        self,
+        ws_callback: Callable[[WebsocketState], None],
+    ) -> None:
+        """Unsubscribe to events websocket state changes."""
+        self._events_ws_state_subscriptions.remove(ws_callback)
+
+    def _unsubscribe_devices_websocket_state(
+        self,
+        ws_callback: Callable[[WebsocketState], None],
+    ) -> None:
+        """Unsubscribe to devices websocket state changes."""
+        self._devices_ws_state_subscriptions.remove(ws_callback)
+
     def _on_websocket_state_change(self, state: WebsocketState) -> None:
         """Websocket state changed."""
         super()._on_websocket_state_change(state)
@@ -1289,6 +1699,26 @@ class ProtectApiClient(BaseApiClient):
             except Exception:
                 _LOGGER.exception("Exception while running websocket state handler")
 
+    def _on_events_websocket_state_change(self, state: WebsocketState) -> None:
+        """Events Websocket state changed."""
+        for sub in self._events_ws_state_subscriptions:
+            try:
+                sub(state)
+            except Exception:
+                _LOGGER.exception(
+                    "Exception while running events websocket state handler"
+                )
+
+    def _on_devices_websocket_state_change(self, state: WebsocketState) -> None:
+        """Devices Websocket state changed."""
+        for sub in self._devices_ws_state_subscriptions:
+            try:
+                sub(state)
+            except Exception:
+                _LOGGER.exception(
+                    "Exception while running devices websocket state handler"
+                )
+
     async def get_bootstrap(self) -> Bootstrap:
         """
         Gets bootstrap object from UFP instance
@@ -2263,3 +2693,42 @@ class ProtectApiClient(BaseApiClient):
         if not isinstance(data, dict):
             raise NvrError("Failed to retrieve meta info from public API")
         return MetaInfo(**data)
+
+    # Public API Methods
+
+    async def get_nvr_public(self) -> NVR:
+        """Get NVR information using public API."""
+        data = await self.api_request_obj(url="/v1/nvrs", public_api=True)
+        return NVR.from_unifi_dict(**data, api=self)
+
+    async def get_lights_public(self) -> list[Light]:
+        """Get all lights using public API."""
+        data = await self.api_request_list(url="/v1/lights", public_api=True)
+        return [Light.from_unifi_dict(**light_data, api=self) for light_data in data]
+
+    async def get_light_public(self, light_id: str) -> Light:
+        """Get a specific light using public API."""
+        data = await self.api_request_obj(url=f"/v1/lights/{light_id}", public_api=True)
+        return Light.from_unifi_dict(**data, api=self)
+
+    async def get_cameras_public(self) -> list[Camera]:
+        """Get all cameras using public API."""
+        data = await self.api_request_list(url="/v1/cameras", public_api=True)
+        return [Camera.from_unifi_dict(**camera_data, api=self) for camera_data in data]
+
+    async def get_camera_public(self, camera_id: str) -> Camera:
+        """Get a specific camera using public API."""
+        data = await self.api_request_obj(
+            url=f"/v1/cameras/{camera_id}", public_api=True
+        )
+        return Camera.from_unifi_dict(**data, api=self)
+
+    async def get_chimes_public(self) -> list[Chime]:
+        """Get all chimes using public API."""
+        data = await self.api_request_list(url="/v1/chimes", public_api=True)
+        return [Chime.from_unifi_dict(**chime_data, api=self) for chime_data in data]
+
+    async def get_chime_public(self, chime_id: str) -> Chime:
+        """Get a specific chime using public API."""
+        data = await self.api_request_obj(url=f"/v1/chimes/{chime_id}", public_api=True)
+        return Chime.from_unifi_dict(**data, api=self)
