@@ -8,9 +8,14 @@ from collections.abc import Callable
 from datetime import datetime, timedelta
 from functools import cache
 from ipaddress import IPv4Address
-from typing import TYPE_CHECKING, Any, ClassVar, TypeVar
+from typing import TYPE_CHECKING, Any, NamedTuple
 from uuid import UUID
 
+from convertertools import pop_dict_set_if_none, pop_dict_tuple
+from pydantic import BaseModel, ConfigDict
+from pydantic.fields import PrivateAttr
+
+from .._compat import cached_property
 from ..exceptions import BadRequest, ClientError, NotAuthorized
 from ..utils import (
     asyncio_timeout,
@@ -27,6 +32,7 @@ from .types import (
     PermissionNode,
     ProtectWSPayloadFormat,
     StateType,
+    get_field_type,
 )
 from .websocket import (
     WSJSONPacketFrame,
@@ -34,17 +40,6 @@ from .websocket import (
     WSPacketFrameHeader,
 )
 
-try:
-    from pydantic.v1 import BaseModel
-    from pydantic.v1.fields import SHAPE_DICT, SHAPE_LIST, PrivateAttr
-except ImportError:
-    from pydantic import BaseModel  # type: ignore[assignment, no-redef]
-    from pydantic.fields import (  # type: ignore[attr-defined, assignment, no-redef]
-        SHAPE_DICT,
-        SHAPE_LIST,
-        PrivateAttr,
-    )
-
 if TYPE_CHECKING:
     from asyncio.events import TimerHandle
 
@@ -55,28 +50,40 @@ if TYPE_CHECKING:
     from ..data.nvr import Event
     from ..data.user import User
 
-    try:
-        from pydantic.v1.typing import DictStrAny
-    except ImportError:
-        from pydantic.typing import (  # type: ignore[assignment, no-redef]
-            DictStrAny,
-        )
 
-
-ProtectObject = TypeVar("ProtectObject", bound="ProtectBaseObject")
 RECENT_EVENT_MAX = timedelta(seconds=30)
 EVENT_PING_INTERVAL = timedelta(seconds=3)
 EVENT_PING_INTERVAL_SECONDS = EVENT_PING_INTERVAL.total_seconds()
 
+_EMPTY_EVENT_PING_BACK: dict[Any, Any] = {}
+
+
 _LOGGER = logging.getLogger(__name__)
 
 
 @cache
-def _is_protect_base_object(cls: type) -> bool:
+def _is_protect_base_object(cls: type[Any]) -> bool:
     """A cached version of `issubclass(cls, ProtectBaseObject)` to speed up the check."""
     return issubclass(cls, ProtectBaseObject)
 
 
+class _ProtectModelObjects(NamedTuple):
+    """
+    Class to track all child of UFP objects.
+
+    objs are UFP objects
+    lists are lists of UFP objects
+    dicts are dicts of UFP objects
+    """
+
+    objs: dict[str, type[ProtectBaseObject]]
+    has_objs: bool
+    lists: dict[str, type[ProtectBaseObject]]
+    has_lists: bool
+    dicts: dict[str, type[ProtectBaseObject]]
+    has_dicts: bool
+
+
 class ProtectBaseObject(BaseModel):
     """
     Base class for building Python objects from UniFi Protect JSON.
@@ -86,17 +93,8 @@ class ProtectBaseObject(BaseModel):
     * Provides `.unifi_dict` to convert object back into UFP JSON
     """
 
-    _api: ProtectApiClient = PrivateAttr(None)
-
-    _protect_objs: ClassVar[dict[str, type[ProtectBaseObject]] | None] = None
-    _protect_lists: ClassVar[dict[str, type[ProtectBaseObject]] | None] = None
-    _protect_dicts: ClassVar[dict[str, type[ProtectBaseObject]] | None] = None
-    _to_unifi_remaps: ClassVar[DictStrAny | None] = None
-
-    class Config:
-        arbitrary_types_allowed = True
-        validate_assignment = True
-        copy_on_model_validation = "shallow"
+    _api: ProtectApiClient = PrivateAttr(None)  # type: ignore[assignment]
+    model_config = ConfigDict(arbitrary_types_allowed=True, validate_assignment=True)
 
     def __init__(self, api: ProtectApiClient | None = None, **data: Any) -> None:
         """
@@ -136,32 +134,38 @@ class ProtectBaseObject(BaseModel):
             data.pop("api", None)
             return cls(api=api, **data)
 
-        return cls.construct(**data)
+        return cls.model_construct(**data)
 
     @classmethod
-    def construct(cls, _fields_set: set[str] | None = None, **values: Any) -> Self:
+    def model_construct(
+        cls, _fields_set: set[str] | None = None, **values: Any
+    ) -> Self:
         api: ProtectApiClient | None = values.pop("api", None)
-        unifi_objs = cls._get_protect_objs()
-        has_unifi_objs = bool(unifi_objs)
-        unifi_lists = cls._get_protect_lists()
-        has_unifi_lists = bool(unifi_lists)
-        unifi_dicts = cls._get_protect_dicts()
-        has_unifi_dicts = bool(unifi_dicts)
+        (
+            unifi_objs,
+            has_unifi_objs,
+            unifi_lists,
+            has_unifi_lists,
+            unifi_dicts,
+            has_unifi_dicts,
+        ) = cls._get_protect_model()
         for key, value in values.items():
             if has_unifi_objs and key in unifi_objs and isinstance(value, dict):
-                values[key] = unifi_objs[key].construct(**value)
+                values[key] = unifi_objs[key].model_construct(**value)
             elif has_unifi_lists and key in unifi_lists and isinstance(value, list):
                 values[key] = [
-                    unifi_lists[key].construct(**v) if isinstance(v, dict) else v
+                    unifi_lists[key].model_construct(**v) if isinstance(v, dict) else v
                     for v in value
                 ]
             elif has_unifi_dicts and key in unifi_dicts and isinstance(value, dict):
                 values[key] = {
-                    k: unifi_dicts[key].construct(**v) if isinstance(v, dict) else v
+                    k: unifi_dicts[key].model_construct(**v)
+                    if isinstance(v, dict)
+                    else v
                     for k, v in value.items()
                 }
 
-        obj = super().construct(_fields_set=_fields_set, **values)
+        obj = super().model_construct(_fields_set=_fields_set, **values)
         if api is not None:
             obj._api = api
 
@@ -192,11 +196,6 @@ class ProtectBaseObject(BaseModel):
 
     @classmethod
     @cache
-    def _get_unifi_remaps_set(self) -> set[str]:
-        """Helper method to get set of all child UFP objects."""
-        return set(self._get_unifi_remaps())
-
-    @classmethod
     def _get_to_unifi_remaps(cls) -> dict[str, str]:
         """
         Helper method for overriding in child classes for reversing remap UFP
@@ -208,76 +207,41 @@ class ProtectBaseObject(BaseModel):
             "python_name": "ufpJsonName"
         }
         """
-        if cls._to_unifi_remaps is None:
-            cls._to_unifi_remaps = {
-                to_key: from_key for from_key, to_key in cls._get_unifi_remaps().items()
-            }
-
-        return cls._to_unifi_remaps
+        return {
+            to_key: from_key for from_key, to_key in cls._get_unifi_remaps().items()
+        }
 
     @classmethod
-    def _set_protect_subtypes(cls) -> None:
+    @cache
+    def _get_protect_model(cls) -> _ProtectModelObjects:
         """Helper method to detect attrs of current class that are UFP Objects themselves"""
-        cls._protect_objs = {}
-        cls._protect_lists = {}
-        cls._protect_dicts = {}
+        objs: dict[str, type[ProtectBaseObject]] = {}
+        lists: dict[str, type[ProtectBaseObject]] = {}
+        dicts: dict[str, type[ProtectBaseObject]] = {}
 
-        for name, field in cls.__fields__.items():
+        for name, field in cls.model_fields.items():
             try:
-                if _is_protect_base_object(field.type_):
-                    if field.shape == SHAPE_LIST:
-                        cls._protect_lists[name] = field.type_
-                    elif field.shape == SHAPE_DICT:
-                        cls._protect_dicts[name] = field.type_
+                origin, type_ = get_field_type(field.annotation)  # type: ignore[arg-type]
+                if _is_protect_base_object(type_):
+                    if origin is list:
+                        lists[name] = type_
+                    elif origin is dict:
+                        dicts[name] = type_
                     else:
-                        cls._protect_objs[name] = field.type_
+                        objs[name] = type_
             except TypeError:
                 pass
 
-    @classmethod
-    @cache
-    def _get_protect_objs(cls) -> dict[str, type[ProtectBaseObject]]:
-        """Helper method to get all child UFP objects"""
-        if cls._protect_objs is None:
-            cls._set_protect_subtypes()
-            assert cls._protect_objs is not None
-        return cls._protect_objs
+        return _ProtectModelObjects(
+            objs, bool(objs), lists, bool(lists), dicts, bool(dicts)
+        )
 
     @classmethod
     @cache
-    def _get_protect_objs_set(cls) -> set[str]:
-        """Helper method to get all child UFP objects"""
-        return set(cls._get_protect_objs())
-
-    @classmethod
-    @cache
-    def _get_protect_lists(cls) -> dict[str, type[ProtectBaseObject]]:
-        """Helper method to get all child of UFP objects (lists)"""
-        if cls._protect_lists is None:
-            cls._set_protect_subtypes()
-            assert cls._protect_lists is not None
-        return cls._protect_lists
-
-    @classmethod
-    @cache
-    def _get_protect_lists_set(cls) -> set[str]:
-        """Helper method to get all child UFP objects"""
-        return set(cls._get_protect_lists())
-
-    @classmethod
-    @cache
-    def _get_protect_dicts(cls) -> dict[str, type[ProtectBaseObject]]:
-        """Helper method to get all child of UFP objects (dicts)"""
-        if cls._protect_dicts is None:
-            cls._set_protect_subtypes()
-            assert cls._protect_dicts is not None
-        return cls._protect_dicts
-
-    @classmethod
-    @cache
-    def _get_protect_dicts_set(cls) -> set[str]:
-        """Helper method to get all child UFP objects"""
-        return set(cls._get_protect_dicts())
+    def _get_excluded_fields(cls) -> set[str]:
+        """Helper method to get all excluded fields for the current object."""
+        protect_model = cls._get_protect_model()
+        return set(protect_model.objs) | set(protect_model.lists)
 
     @classmethod
     def _clean_protect_obj(
@@ -299,9 +263,7 @@ class ProtectBaseObject(BaseModel):
         klass: type[ProtectBaseObject],
         api: ProtectApiClient | None,
     ) -> list[Any]:
-        for index, item in enumerate(items):
-            items[index] = cls._clean_protect_obj(item, klass, api)
-        return items
+        return [cls._clean_protect_obj(item, klass, api) for item in items]
 
     @classmethod
     def _clean_protect_obj_dict(
@@ -310,9 +272,20 @@ class ProtectBaseObject(BaseModel):
         klass: type[ProtectBaseObject],
         api: ProtectApiClient | None,
     ) -> dict[Any, Any]:
-        for key, value in items.items():
-            items[key] = cls._clean_protect_obj(value, klass, api)
-        return items
+        return {k: cls._clean_protect_obj(v, klass, api) for k, v in items.items()}
+
+    @classmethod
+    @cache
+    def unifi_dict_conversions(cls) -> dict[str, object | Callable[[Any], Any]]:
+        """
+        Helper method for overriding in child classes for converting UFP JSON data to Python data types.
+
+        Return format is
+        {
+            "ufpJsonName": Callable[[Any], Any]
+        }
+        """
+        return {}
 
     @classmethod
     def unifi_dict_to_dict(cls, data: dict[str, Any]) -> dict[str, Any]:
@@ -334,38 +307,48 @@ class ProtectBaseObject(BaseModel):
             cls._api if isinstance(cls, ProtectBaseObject) else None
         )
 
+        conversions = cls.unifi_dict_conversions()
+        for key, convert in conversions.items():
+            if (val := data.get(key)) is not None:
+                data[key] = convert(val)  # type: ignore[operator]
+
         remaps = cls._get_unifi_remaps()
         # convert to snake_case and remove extra fields
-        _fields = cls.__fields__
-        for key in list(data):
-            if key in remaps:
+        _fields = cls.model_fields
+        for key in data.copy():
+            current_key = key
+            if current_key in remaps:
                 # remap keys that will not be converted correctly by snake_case convert
-                remapped_key = remaps[key]
-                data[remapped_key] = data.pop(key)
-                key = remapped_key
+                remapped_key = remaps[current_key]
+                data[remapped_key] = data.pop(current_key)
+                current_key = remapped_key
 
-            new_key = to_snake_case(key)
-            data[new_key] = data.pop(key)
-            key = new_key
+            new_key = to_snake_case(current_key)
+            data[new_key] = data.pop(current_key)
+            current_key = new_key
 
-            if key == "api":
+            if current_key == "api":
                 continue
 
-            if key not in _fields:
-                del data[key]
+            if current_key not in _fields:
+                del data[current_key]
                 continue
-            data[key] = convert_unifi_data(data[key], _fields[key])
+            data[current_key] = convert_unifi_data(
+                data[current_key], _fields[current_key]
+            )
 
         if not data:
             return data
 
         # clean child UFP objs
-        unifi_objs = cls._get_protect_objs()
-        has_unifi_objs = bool(unifi_objs)
-        unifi_lists = cls._get_protect_lists()
-        has_unifi_lists = bool(unifi_lists)
-        unifi_dicts = cls._get_protect_dicts()
-        has_unifi_dicts = bool(unifi_dicts)
+        (
+            unifi_objs,
+            has_unifi_objs,
+            unifi_lists,
+            has_unifi_lists,
+            unifi_dicts,
+            has_unifi_dicts,
+        ) = cls._get_protect_model()
         for key, value in data.items():
             if has_unifi_objs and key in unifi_objs:
                 data[key] = cls._clean_protect_obj(value, unifi_objs[key], api)
@@ -390,7 +373,7 @@ class ProtectBaseObject(BaseModel):
         if isinstance(value, ProtectBaseObject):
             value = value.unifi_dict()
         elif isinstance(value, dict):
-            value = klass.construct({}).unifi_dict(data=value)  # type: ignore[arg-type]
+            value = klass.model_construct({}).unifi_dict(data=value)  # type: ignore[arg-type]
 
         return value
 
@@ -408,15 +391,12 @@ class ProtectBaseObject(BaseModel):
         if not isinstance(value, list):
             return value
 
-        items: list[Any] = []
-        for item in value:
-            if isinstance(item, ProtectBaseObject):
-                new_item = item.unifi_dict()
-            else:
-                new_item = klass.construct({}).unifi_dict(data=item)  # type: ignore[arg-type]
-            items.append(new_item)
-
-        return items
+        return [
+            item.unifi_dict()
+            if isinstance(item, ProtectBaseObject)
+            else klass.model_construct({}).unifi_dict(data=item)  # type: ignore[arg-type]
+            for item in value
+        ]
 
     def _unifi_dict_protect_obj_dict(
         self,
@@ -431,13 +411,10 @@ class ProtectBaseObject(BaseModel):
         if not isinstance(value, dict):
             return value
 
-        items: dict[Any, Any] = {}
-        for obj_key, obj in value.items():
-            if isinstance(obj, ProtectBaseObject):
-                obj = obj.unifi_dict()
-            items[obj_key] = obj
-
-        return items
+        return {
+            obj_key: obj.unifi_dict() if isinstance(obj, ProtectBaseObject) else obj
+            for obj_key, obj in value.items()
+        }
 
     def unifi_dict(
         self,
@@ -461,25 +438,37 @@ class ProtectBaseObject(BaseModel):
         """
         use_obj = False
         if data is None:
-            excluded_fields = (
-                self._get_protect_objs_set() | self._get_protect_lists_set()
-            )
+            excluded_fields = self._get_excluded_fields()
             if exclude is not None:
-                excluded_fields |= exclude
-            data = self.dict(exclude=excluded_fields)
+                excluded_fields = excluded_fields.copy() | exclude
+            data = self.model_dump(exclude=excluded_fields)
             use_obj = True
 
-        for key, klass in self._get_protect_objs().items():
-            if use_obj or key in data:
-                data[key] = self._unifi_dict_protect_obj(data, key, use_obj, klass)
+        (
+            unifi_objs,
+            has_unifi_objs,
+            unifi_lists,
+            has_unifi_lists,
+            unifi_dicts,
+            has_unifi_dicts,
+        ) = self._get_protect_model()
 
-        for key, klass in self._get_protect_lists().items():
-            if use_obj or key in data:
-                data[key] = self._unifi_dict_protect_obj_list(data, key, use_obj, klass)
+        if has_unifi_objs:
+            for key, klass in unifi_objs.items():
+                if use_obj or key in data:
+                    data[key] = self._unifi_dict_protect_obj(data, key, use_obj, klass)
 
-        for key in self._get_protect_dicts():
-            if use_obj or key in data:
-                data[key] = self._unifi_dict_protect_obj_dict(data, key, use_obj)
+        if has_unifi_lists:
+            for key, klass in unifi_lists.items():
+                if use_obj or key in data:
+                    data[key] = self._unifi_dict_protect_obj_list(
+                        data, key, use_obj, klass
+                    )
+
+        if has_unifi_dicts:
+            for key in unifi_dicts:
+                if use_obj or key in data:
+                    data[key] = self._unifi_dict_protect_obj_dict(data, key, use_obj)
 
         # all child objects have been serialized correctly do not do it twice
         new_data: dict[str, Any] = serialize_unifi_obj(data, levels=2)
@@ -487,81 +476,55 @@ class ProtectBaseObject(BaseModel):
         for to_key in set(new_data).intersection(remaps):
             new_data[remaps[to_key]] = new_data.pop(to_key)
 
-        if "api" in new_data:
-            del new_data["api"]
-
         return new_data
 
-    def _inject_api(
-        self,
-        data: dict[str, Any],
-        api: ProtectApiClient | None,
-    ) -> dict[str, Any]:
-        data["api"] = api
-        unifi_objs_sets = self._get_protect_objs_set()
-        has_unifi_objs = bool(unifi_objs_sets)
-        unifi_lists_sets = self._get_protect_lists_set()
-        has_unifi_lists = bool(unifi_lists_sets)
-        unifi_dicts_sets = self._get_protect_dicts_set()
-        has_unifi_dicts = bool(unifi_dicts_sets)
-        for key, value in data.items():
-            if has_unifi_objs and key in unifi_objs_sets and isinstance(value, dict):
-                value["api"] = api
-            elif (
-                has_unifi_lists and key in unifi_lists_sets and isinstance(value, list)
-            ):
-                for item in value:
-                    if isinstance(item, dict):
-                        item["api"] = api
-            elif (
-                has_unifi_dicts and key in unifi_dicts_sets and isinstance(value, dict)
-            ):
-                for item in value.values():
-                    if isinstance(item, dict):
-                        item["api"] = api
+    def update_from_dict(self, data: dict[str, Any]) -> Self:
+        """
+        Updates current object from a cleaned UFP JSON dict.
 
-        return data
+        The api client is injected into each dict for any child
+        UFP objects that are detected.
+        """
+        (
+            unifi_objs,
+            has_unifi_objs,
+            unifi_lists,
+            has_unifi_lists,
+            _unifi_dicts,
+            _has_unifi_dicts,
+        ) = self._get_protect_model()
+        api = self._api
+        _fields = self.__class__.model_fields
+        unifi_obj: ProtectBaseObject | None
+        value: Any
 
-    def update_from_dict(self: ProtectObject, data: dict[str, Any]) -> ProtectObject:
-        """Updates current object from a cleaned UFP JSON dict"""
-        data_set = set(data)
-        for key in self._get_protect_objs_set().intersection(data_set):
-            unifi_obj: Any | None = getattr(self, key)
-            if unifi_obj is not None and isinstance(unifi_obj, ProtectBaseObject):
-                item = data.pop(key)
-                if item is not None:
-                    item = unifi_obj.update_from_dict(item)
-                setattr(self, key, item)
+        for key, item in data.items():
+            if has_unifi_objs and key in unifi_objs and isinstance(item, dict):
+                if (unifi_obj := getattr(self, key)) is not None:
+                    value = unifi_obj.update_from_dict(item)
+                else:
+                    value = unifi_objs[key](**item, api=api)
+            elif has_unifi_lists and key in unifi_lists and isinstance(item, list):
+                klass = unifi_lists[key]
+                value = [
+                    klass(**i, api=api) if isinstance(i, dict) else i
+                    for i in item
+                    if i is not None and isinstance(i, (dict, ProtectBaseObject))
+                ]
+            else:
+                value = convert_unifi_data(item, _fields[key])
 
-        data = self._inject_api(data, self._api)
-        unifi_lists = self._get_protect_lists()
-        for key in self._get_protect_lists_set().intersection(data_set):
-            if not isinstance(data[key], list):
-                continue
-            klass = unifi_lists[key]
-            new_items = []
-            for item in data.pop(key):
-                if item is not None and isinstance(item, ProtectBaseObject):
-                    new_items.append(item)
-                elif isinstance(item, dict):
-                    new_items.append(klass(**item))
-            setattr(self, key, new_items)
-
-        # Always injected above
-        del data["api"]
-
-        for key in data:
-            setattr(self, key, convert_unifi_data(data[key], self.__fields__[key]))
+            setattr(self, key, value)
 
         return self
 
     def dict_with_excludes(self) -> dict[str, Any]:
         """Returns a dict of the current object without any UFP objects converted to dicts."""
         excludes = self.__class__._get_excluded_changed_fields()
-        return self.dict(exclude=excludes)
+        return self.model_dump(exclude=excludes)
 
     def get_changed(self, data_before_changes: dict[str, Any]) -> dict[str, Any]:
-        return dict_diff(data_before_changes, self.dict())
+        return dict_diff(data_before_changes, self.model_dump())
 
     @property
     def api(self) -> ProtectApiClient:
@@ -581,7 +544,7 @@ class ProtectModel(ProtectBaseObject):
     automatically decoding a `modelKey` object into the correct UFP object and type
     """
 
-    model: ModelType | None
+    model: ModelType | None = None
 
     @classmethod
     @cache
@@ -594,42 +557,50 @@ class ProtectModel(ProtectBaseObject):
         exclude: set[str] | None = None,
     ) -> dict[str, Any]:
         data = super().unifi_dict(data=data, exclude=exclude)
-
-        if "modelKey" in data and data["modelKey"] is None:
-            del data["modelKey"]
-
+        pop_dict_set_if_none(data, {"modelKey"})
         return data
 
 
+class UpdateSynchronization:
+    """Helper class for managing updates to Protect devices."""
+
+    @cached_property
+    def lock(self) -> asyncio.Lock:
+        """Lock to prevent multiple updates at once."""
+        return asyncio.Lock()
+
+    @cached_property
+    def queue(self) -> asyncio.Queue[Callable[[], None]]:
+        """Queue to store device updates."""
+        return asyncio.Queue()
+
+    @cached_property
+    def event(self) -> asyncio.Event:
+        """Event to signal when a device update has been queued."""
+        return asyncio.Event()
+
+
 class ProtectModelWithId(ProtectModel):
     id: str
 
-    _update_lock: asyncio.Lock = PrivateAttr(None)
-    _update_queue: asyncio.Queue[Callable[[], None]] = PrivateAttr(None)
-    _update_event: asyncio.Event = PrivateAttr(None)
+    _update_sync: UpdateSynchronization = PrivateAttr(None)  # type: ignore[assignment]
 
     def __init__(self, **data: Any) -> None:
-        update_lock = data.pop("update_lock", None)
-        update_queue = data.pop("update_queue", None)
-        update_event = data.pop("update_event", None)
+        update_sync = data.pop("update_sync", None)
         super().__init__(**data)
-        self._update_lock = update_lock or asyncio.Lock()
-        self._update_queue = update_queue or asyncio.Queue()
-        self._update_event = update_event or asyncio.Event()
+        self._update_sync = update_sync or UpdateSynchronization()
 
     @classmethod
-    def construct(cls, _fields_set: set[str] | None = None, **values: Any) -> Self:
-        update_lock = values.pop("update_lock", None)
-        update_queue = values.pop("update_queue", None)
-        update_event = values.pop("update_event", None)
-        obj = super().construct(_fields_set=_fields_set, **values)
-        obj._update_lock = update_lock or asyncio.Lock()
-        obj._update_queue = update_queue or asyncio.Queue()
-        obj._update_event = update_event or asyncio.Event()
-
+    def model_construct(
+        cls, _fields_set: set[str] | None = None, **values: Any
+    ) -> Self:
+        update_sync = values.pop("update_sync", None)
+        obj = super().model_construct(_fields_set=_fields_set, **values)
+        obj._update_sync = update_sync or UpdateSynchronization()
         return obj
 
     @classmethod
+    @cache
     def _get_read_only_fields(cls) -> set[str]:
         return set()
 
@@ -643,28 +614,24 @@ class ProtectModelWithId(ProtectModel):
             setattr(self, key, data_before_changes[key])
 
     def can_create(self, user: User) -> bool:
-        if self.model is None:
-            return True
-
-        return user.can(self.model, PermissionNode.CREATE, self)
+        if (model := self.model) is not None:
+            return user.can(model, PermissionNode.CREATE, self)
+        return True
 
     def can_read(self, user: User) -> bool:
-        if self.model is None:
-            return True
-
-        return user.can(self.model, PermissionNode.READ, self)
+        if (model := self.model) is not None:
+            return user.can(model, PermissionNode.READ, self)
+        return True
 
     def can_write(self, user: User) -> bool:
-        if self.model is None:
-            return True
-
-        return user.can(self.model, PermissionNode.WRITE, self)
+        if (model := self.model) is not None:
+            return user.can(model, PermissionNode.WRITE, self)
+        return True
 
     def can_delete(self, user: User) -> bool:
-        if self.model is None:
-            return True
-
-        return user.can(self.model, PermissionNode.DELETE, self)
+        if (model := self.model) is not None:
+            return user.can(model, PermissionNode.DELETE, self)
+        return True
 
     async def queue_update(self, callback: Callable[[], None]) -> None:
         """
@@ -673,28 +640,27 @@ class ProtectModelWithId(ProtectModel):
         This allows aggregating devices updates so if multiple ones come in all at once,
         they can be combined in a single PATCH.
         """
-        self._update_queue.put_nowait(callback)
+        self._update_sync.queue.put_nowait(callback)
 
-        self._update_event.set()
-        await asyncio.sleep(
-            0.001,
-        )  # release execution so other `queue_update` calls can abort
-        self._update_event.clear()
+        self._update_sync.event.set()
+        # release execution so other `queue_update` calls can abort
+        await asyncio.sleep(0.001)
+        self._update_sync.event.clear()
 
         try:
             async with asyncio_timeout(0.05):
-                await self._update_event.wait()
-            self._update_event.clear()
+                await self._update_sync.event.wait()
+            self._update_sync.event.clear()
             return
-        except (TimeoutError, asyncio.TimeoutError, asyncio.CancelledError):
-            async with self._update_lock:
+        except (TimeoutError, asyncio.TimeoutError):
+            async with self._update_sync.lock:
                 # Important! Now that we have the lock, we yield to the event loop so any
                 # updates from the websocket are processed before we generate the diff
                 await asyncio.sleep(0)
                 # Save the initial data before we generate the diff
                 data_before_changes = self.dict_with_excludes()
-                while not self._update_queue.empty():
-                    callback = self._update_queue.get_nowait()
+                while not self._update_sync.queue.empty():
+                    callback = self._update_sync.queue.get_nowait()
                     callback()
                 # Important, do not yield to the event loop before generating the diff
                 # otherwise we may miss updates from the websocket
@@ -724,8 +690,8 @@ class ProtectModelWithId(ProtectModel):
         """
         # do not allow multiple save_device calls at once
         release_lock = False
-        if not self._update_lock.locked():
-            await self._update_lock.acquire()
+        if not self._update_sync.lock.locked():
+            await self._update_sync.lock.acquire()
             release_lock = True
 
         try:
@@ -737,7 +703,7 @@ class ProtectModelWithId(ProtectModel):
             )
         finally:
             if release_lock:
-                self._update_lock.release()
+                self._update_sync.lock.release()
 
     async def _save_device_changes(
         self,
@@ -755,9 +721,9 @@ class ProtectModelWithId(ProtectModel):
             updated,
         )
 
-        assert (
-            self._update_lock.locked()
-        ), "save_device_changes should only be called when the update lock is held"
+        assert self._update_sync.lock.locked(), (
+            "save_device_changes should only be called when the update lock is held"
+        )
         read_only_fields = self.__class__._get_read_only_fields()
 
         if self.model is None:
@@ -797,7 +763,7 @@ class ProtectModelWithId(ProtectModel):
 
     def _emit_message(self, updated: dict[str, Any]) -> None:
         """Emits fake WS message for ProtectApiClient to process."""
-        if updated == {}:
+        if _is_ping_back := updated is _EMPTY_EVENT_PING_BACK:
             _LOGGER.debug("Event ping callback started for %s", self.id)
 
         if self.model is None:
@@ -826,21 +792,23 @@ class ProtectModelWithId(ProtectModel):
 
         message = self._api.bootstrap.process_ws_packet(
             WSPacket(action_frame.packed + data_frame.packed),
+            is_ping_back=_is_ping_back,
         )
+
         if message is not None:
             self._api.emit_message(message)
 
 
 class ProtectDeviceModel(ProtectModelWithId):
-    name: str | None
+    name: str | None = None
     type: str
     mac: str
-    host: IPv4Address | str | None
-    up_since: datetime | None
-    uptime: timedelta | None
-    last_seen: datetime | None
-    hardware_revision: str | None
-    firmware_version: str | None
+    host: IPv4Address | str | None = None
+    up_since: datetime | None = None
+    uptime: timedelta | None = None
+    last_seen: datetime | None = None
+    hardware_revision: str | None = None
+    firmware_version: str | None = None
     is_updating: bool
     is_ssh_enabled: bool
 
@@ -861,23 +829,16 @@ class ProtectDeviceModel(ProtectModelWithId):
         }
 
     @classmethod
-    def unifi_dict_to_dict(cls, data: dict[str, Any]) -> dict[str, Any]:
-        if "lastSeen" in data:
-            data["lastSeen"] = convert_to_datetime(data["lastSeen"])
-        if "upSince" in data and data["upSince"] is not None:
-            data["upSince"] = convert_to_datetime(data["upSince"])
-        if (
-            "uptime" in data
-            and data["uptime"] is not None
-            and not isinstance(data["uptime"], timedelta)
-        ):
-            data["uptime"] = timedelta(milliseconds=int(data["uptime"]))
-        # hardware revisions for all devices are not simple numbers
-        # so cast them all to str to be consistent
-        if "hardwareRevision" in data and data["hardwareRevision"] is not None:
-            data["hardwareRevision"] = str(data["hardwareRevision"])
-
-        return super().unifi_dict_to_dict(data)
+    @cache
+    def unifi_dict_conversions(cls) -> dict[str, object | Callable[[Any], Any]]:
+        return {
+            "upSince": convert_to_datetime,
+            "uptime": lambda x: timedelta(milliseconds=int(x)),
+            "lastSeen": convert_to_datetime,
+            # hardware revisions for all devices are not simple numbers
+            # so cast them all to str to be consistent
+            "hardwareRevision": str,
+        } | super().unifi_dict_conversions()
 
     def _event_callback_ping(self) -> None:
         _LOGGER.debug("Event ping timer started for %s", self.id)
@@ -885,7 +846,7 @@ class ProtectDeviceModel(ProtectModelWithId):
         self._callback_ping = loop.call_later(
             EVENT_PING_INTERVAL_SECONDS,
             self._emit_message,
-            {},
+            _EMPTY_EVENT_PING_BACK,
         )
 
     async def set_name(self, name: str | None) -> None:
@@ -898,12 +859,12 @@ class ProtectDeviceModel(ProtectModelWithId):
 
 
 class WiredConnectionState(ProtectBaseObject):
-    phy_rate: float | None
+    phy_rate: float | None = None
 
 
 class WirelessConnectionState(ProtectBaseObject):
-    signal_quality: int | None
-    signal_strength: int | None
+    signal_quality: int | None = None
+    signal_strength: int | None = None
 
 
 class BluetoothConnectionState(WirelessConnectionState):
@@ -911,10 +872,10 @@ class BluetoothConnectionState(WirelessConnectionState):
 
 
 class WifiConnectionState(WirelessConnectionState):
-    phy_rate: float | None
-    channel: int | None
-    frequency: int | None
-    ssid: str | None
+    phy_rate: float | None = None
+    channel: int | None = None
+    frequency: int | None = None
+    ssid: str | None = None
     bssid: str | None = None
     tx_rate: float | None = None
     # requires 2.7.5+
@@ -926,10 +887,10 @@ class WifiConnectionState(WirelessConnectionState):
 
 class ProtectAdoptableDeviceModel(ProtectDeviceModel):
     state: StateType
-    connection_host: IPv4Address | str | None
-    connected_since: datetime | None
-    latest_firmware_version: str | None
-    firmware_build: str | None
+    connection_host: IPv4Address | str | None = None
+    connected_since: datetime | None = None
+    latest_firmware_version: str | None = None
+    firmware_build: str | None = None
     is_adopting: bool
     is_adopted: bool
     is_adopted_by_other: bool
@@ -939,7 +900,7 @@ class ProtectAdoptableDeviceModel(ProtectDeviceModel):
     is_attempting_to_connect: bool
     is_connected: bool
     # requires 1.21+
-    market_name: str | None
+    market_name: str | None = None
     # requires 2.7.5+
     fw_update_state: str | None = None
     # requires 2.8.14+
@@ -954,8 +915,8 @@ class ProtectAdoptableDeviceModel(ProtectDeviceModel):
     wired_connection_state: WiredConnectionState | None = None
     wifi_connection_state: WifiConnectionState | None = None
     bluetooth_connection_state: BluetoothConnectionState | None = None
-    bridge_id: str | None
-    is_downloading_firmware: bool | None
+    bridge_id: str | None = None
+    is_downloading_firmware: bool | None = None
 
     # TODO:
     # bridgeCandidates
@@ -989,8 +950,8 @@ class ProtectAdoptableDeviceModel(ProtectDeviceModel):
         }
 
     async def _api_update(self, data: dict[str, Any]) -> None:
-        if self.model is not None:
-            return await self._api.update_device(self.model, self.id, data)
+        if (model := self.model) is not None:
+            return await self._api.update_device(model, self.id, data)
         return None
 
     def unifi_dict(
@@ -999,24 +960,18 @@ class ProtectAdoptableDeviceModel(ProtectDeviceModel):
         exclude: set[str] | None = None,
     ) -> dict[str, Any]:
         data = super().unifi_dict(data=data, exclude=exclude)
-
-        if "wiredConnectionState" in data and data["wiredConnectionState"] is None:
-            del data["wiredConnectionState"]
-        if "wifiConnectionState" in data and data["wifiConnectionState"] is None:
-            del data["wifiConnectionState"]
-        if (
-            "bluetoothConnectionState" in data
-            and data["bluetoothConnectionState"] is None
-        ):
-            del data["bluetoothConnectionState"]
+        pop_dict_set_if_none(
+            data,
+            {"wiredConnectionState", "wifiConnectionState", "bluetoothConnectionState"},
+        )
         return data
 
     @classmethod
-    def unifi_dict_to_dict(cls, data: dict[str, Any]) -> dict[str, Any]:
-        if "lastDisconnect" in data and data["lastDisconnect"] is not None:
-            data["lastDisconnect"] = convert_to_datetime(data["lastDisconnect"])
-
-        return super().unifi_dict_to_dict(data)
+    @cache
+    def unifi_dict_conversions(cls) -> dict[str, object | Callable[[Any], Any]]:
+        return {
+            "lastDisconnect": convert_to_datetime,
+        } | super().unifi_dict_conversions()
 
     @property
     def display_name(self) -> str:
@@ -1036,10 +991,9 @@ class ProtectAdoptableDeviceModel(ProtectDeviceModel):
 
     @property
     def bridge(self) -> Bridge | None:
-        if self.bridge_id is None:
-            return None
-
-        return self._api.bootstrap.bridges[self.bridge_id]
+        if (bridge_id := self.bridge_id) is not None:
+            return self._api.bootstrap.bridges[bridge_id]
+        return None
 
     @property
     def protect_url(self) -> str:
@@ -1103,7 +1057,7 @@ class ProtectAdoptableDeviceModel(ProtectDeviceModel):
 
 
 class ProtectMotionDeviceModel(ProtectAdoptableDeviceModel):
-    last_motion: datetime | None
+    last_motion: datetime | None = None
     is_dark: bool
 
     # not directly from UniFi
@@ -1120,15 +1074,11 @@ class ProtectMotionDeviceModel(ProtectAdoptableDeviceModel):
         exclude: set[str] | None = None,
     ) -> dict[str, Any]:
         data = super().unifi_dict(data=data, exclude=exclude)
-
-        if "lastMotionEventId" in data:
-            del data["lastMotionEventId"]
-
+        pop_dict_tuple(data, ("lastMotionEventId",))
         return data
 
     @property
     def last_motion_event(self) -> Event | None:
-        if self.last_motion_event_id is None:
-            return None
-
-        return self._api.bootstrap.events.get(self.last_motion_event_id)
+        if (last_motion_event_id := self.last_motion_event_id) is not None:
+            return self._api.bootstrap.events.get(last_motion_event_id)
+        return None
