@@ -1,28 +1,52 @@
 from __future__ import annotations
 
 import enum
-from collections.abc import Callable, Coroutine
-from functools import cache, cached_property
-from typing import Any, Literal, Optional, TypeVar, Union
+import types
+from collections.abc import Callable, Coroutine, Sequence
+from functools import cache, lru_cache
+from typing import Annotated, Any, Literal, TypeVar, Union, get_args, get_origin
 
 from packaging.version import Version as BaseVersion
+from pydantic import BaseModel, Field
+from pydantic.types import StringConstraints
+from pydantic_extra_types.color import Color  # noqa: F401
 
-try:
-    from pydantic.v1 import BaseModel, ConstrainedInt
-    from pydantic.v1.color import Color as BaseColor
-    from pydantic.v1.types import ConstrainedFloat, ConstrainedStr
-except ImportError:
-    from pydantic import BaseModel, ConstrainedInt  # type: ignore[assignment, no-redef]
-    from pydantic.color import Color as BaseColor  # type: ignore[assignment, no-redef]
-    from pydantic.types import (  # type: ignore[assignment, no-redef]
-        ConstrainedFloat,
-        ConstrainedStr,
-    )
+from .._compat import cached_property
 
 KT = TypeVar("KT")
 VT = TypeVar("VT")
 
 
+@lru_cache(maxsize=512)
+def get_field_type(annotation: type[Any] | None) -> tuple[type | None, Any]:
+    """Extract the origin and type from an annotation."""
+    if annotation is None:
+        raise ValueError("Type annotation cannot be None")
+    origin = get_origin(annotation)
+    args: Sequence[Any]
+    if origin in (list, set):
+        if not (args := get_args(annotation)):
+            raise ValueError(f"Unable to determine args of type: {annotation}")
+        return origin, args[0]
+    if origin is dict:
+        if not (args := get_args(annotation)):
+            raise ValueError(f"Unable to determine args of type: {annotation}")
+        return origin, args[1]
+    if origin is Annotated:
+        if not (args := get_args(annotation)):
+            raise ValueError(f"Unable to determine args of type: {annotation}")
+        return None, args[0]
+    if origin is Union or origin is types.UnionType:
+        if not (args := get_args(annotation)):
+            raise ValueError(f"Unable to determine args of type: {annotation}")
+        args = [get_field_type(arg) for arg in args]
+        if len(args) == 2 and type(None) in list(zip(*args, strict=False))[1]:
+            # Strip '| None' type from Union
+            return next(arg for arg in args if arg[1] is not type(None))
+        return None, annotation
+    return origin, annotation
+
+
 DEFAULT = "DEFAULT_VALUE"
 DEFAULT_TYPE = Literal["DEFAULT_VALUE"]
 EventCategories = Literal[
@@ -36,7 +60,7 @@ EventCategories = Literal[
 ]
 
 ProgressCallback = Callable[[int, int, int], Coroutine[Any, Any, None]]
-IteratorCallback = Callable[[int, Optional[bytes]], Coroutine[Any, Any, None]]
+IteratorCallback = Callable[[int, bytes | None], Coroutine[Any, Any, None]]
 
 
 class FixSizeOrderedDict(dict[KT, VT]):
@@ -58,6 +82,11 @@ class ValuesEnumMixin:
     _values: list[str] | None = None
     _values_normalized: dict[str, str] | None = None
 
+    @classmethod
+    @cache
+    def from_string(cls, value: str) -> Any:
+        return cls(value)  # type: ignore[call-arg]
+
     @classmethod
     @cache
     def values(cls) -> list[str]:
@@ -105,23 +134,42 @@ class ModelType(str, UnknownValuesEnumMixin, enum.Enum):
     DOORLOCK = "doorlock"
     SCHEDULE = "schedule"
     CHIME = "chime"
+    AIPORT = "aiport"
     DEVICE_GROUP = "deviceGroup"
     RECORDING_SCHEDULE = "recordingSchedule"
+    ULP_USER = "ulpUser"
+    RINGTONE = "ringtone"
+    KEYRING = "keyring"
     UNKNOWN = "unknown"
 
+    bootstrap_model_types: tuple[ModelType, ...]
+    bootstrap_models: tuple[str, ...]
+    bootstrap_models_set: set[str]
+    bootstrap_models_types_set: set[ModelType]
+    bootstrap_models_types_and_event_set: set[ModelType]
+
     @cached_property
     def devices_key(self) -> str:
         """Return the devices key."""
         return f"{self.value}s"
 
+    @cached_property
+    def name(self) -> str:
+        """Return the name."""
+        return self._name_
+
+    @cached_property
+    def value(self) -> str:
+        """Return the value."""
+        return self._value_
+
     @classmethod
     @cache
     def from_string(cls, value: str) -> ModelType:
         return cls(value)
 
-    @staticmethod
-    @cache
-    def bootstrap_model_types() -> tuple[ModelType, ...]:
+    @classmethod
+    def _bootstrap_model_types(cls) -> tuple[ModelType, ...]:
         """Return the bootstrap models as a tuple."""
         # TODO:
         # legacyUFV
@@ -137,33 +185,43 @@ class ModelType(str, UnknownValuesEnumMixin, enum.Enum):
             ModelType.SENSOR,
             ModelType.DOORLOCK,
             ModelType.CHIME,
+            ModelType.AIPORT,
         )
 
-    @staticmethod
-    @cache
-    def bootstrap_models() -> tuple[str, ...]:
+    @classmethod
+    def _bootstrap_models(cls) -> tuple[str, ...]:
         """Return the bootstrap models strings as a tuple."""
         return tuple(
-            model_type.value for model_type in ModelType.bootstrap_model_types()
+            model_type.value for model_type in ModelType._bootstrap_model_types()
         )
 
-    @staticmethod
-    @cache
-    def bootstrap_models_set() -> set[str]:
+    @classmethod
+    def _bootstrap_models_set(cls) -> set[str]:
         """Return the set of bootstrap models strings as a set."""
-        return set(ModelType.bootstrap_models())
+        return set(ModelType._bootstrap_models())
 
-    @staticmethod
-    @cache
-    def bootstrap_models_types_set() -> set[ModelType]:
+    @classmethod
+    def _bootstrap_models_types_set(cls) -> set[ModelType]:
         """Return the set of bootstrap models as a set."""
-        return set(ModelType.bootstrap_model_types())
+        return set(ModelType._bootstrap_model_types())
 
-    @staticmethod
-    @cache
-    def bootstrap_models_types_and_event_set() -> set[ModelType]:
+    @classmethod
+    def _bootstrap_models_types_and_event_set(cls) -> set[ModelType]:
         """Return the set of bootstrap models and the event model as a set."""
-        return ModelType.bootstrap_models_types_set() | {ModelType.EVENT}
+        return ModelType._bootstrap_models_types_set() | {ModelType.EVENT}
+
+    def _immutable(self, name: str, value: Any) -> None:
+        raise AttributeError("Cannot modify ModelType")
+
+
+ModelType.bootstrap_model_types = ModelType._bootstrap_model_types()
+ModelType.bootstrap_models = ModelType._bootstrap_models()
+ModelType.bootstrap_models_set = ModelType._bootstrap_models_set()
+ModelType.bootstrap_models_types_set = ModelType._bootstrap_models_types_set()
+ModelType.bootstrap_models_types_and_event_set = (
+    ModelType._bootstrap_models_types_and_event_set()
+)
+ModelType.__setattr__ = ModelType._immutable  # type: ignore[method-assign, assignment]
 
 
 @enum.unique
@@ -179,6 +237,8 @@ class EventType(str, ValuesEnumMixin, enum.Enum):
     POOR_CONNECTION = "poorConnection"
     STREAM_RECOVERY = "streamRecovery"
     MOTION = "motion"
+    NFC_CARD_SCANNED = "nfcCardScanned"
+    FINGERPRINT_IDENTIFIED = "fingerprintIdentified"
     RECORDING_DELETED = "recordingDeleted"
     SMART_AUDIO_DETECT = "smartAudioDetect"
     SMART_DETECT = "smartDetectZone"
@@ -252,8 +312,12 @@ class EventType(str, ValuesEnumMixin, enum.Enum):
     def device_events() -> list[str]:
         return [
             EventType.MOTION.value,
+            EventType.NFC_CARD_SCANNED.value,
+            EventType.FINGERPRINT_IDENTIFIED.value,
             EventType.RING.value,
             EventType.SMART_DETECT.value,
+            EventType.SMART_AUDIO_DETECT.value,
+            EventType.SMART_DETECT_LINE.value,
         ]
 
     @staticmethod
@@ -381,6 +445,8 @@ class VideoMode(str, ValuesEnumMixin, enum.Enum):
     HOMEKIT = "homekit"
     SPORT = "sport"
     SLOW_SHUTTER = "slowShutter"
+    LPR_NONE_REFLEX = "lprNoneReflex"
+    LPR_REFLEX = "lprReflex"
     # should only be for unadopted devices
     UNKNOWN = "unknown"
 
@@ -393,6 +459,7 @@ class AudioStyle(str, UnknownValuesEnumMixin, enum.Enum):
 
 @enum.unique
 class RecordingMode(str, ValuesEnumMixin, enum.Enum):
+    ADAPTIVE = "adaptive"
     ALWAYS = "always"
     NEVER = "never"
     SCHEDULE = "schedule"
@@ -470,6 +537,7 @@ class SleepStateType(str, ValuesEnumMixin, enum.Enum):
 class AutoExposureMode(str, ValuesEnumMixin, enum.Enum):
     MANUAL = "manual"
     AUTO = "auto"
+    NONE = "none"
     SHUTTER = "shutter"
     FLICK50 = "flick50"
     FLICK60 = "flick60"
@@ -479,6 +547,7 @@ class AutoExposureMode(str, ValuesEnumMixin, enum.Enum):
 class FocusMode(str, ValuesEnumMixin, enum.Enum):
     MANUAL = "manual"
     AUTO = "auto"
+    NONE = "none"
     ZTRIG = "ztrig"
     TOUCH = "touch"
 
@@ -570,9 +639,20 @@ class PermissionNode(str, UnknownValuesEnumMixin, enum.Enum):
     READ_LIVE = "readlive"
     UNKNOWN = "unknown"
 
+    @cached_property
+    def name(self) -> str:
+        """Return the name."""
+        return self._name_
+
+    @cached_property
+    def value(self) -> str:
+        """Return the value."""
+        return self._value_
+
 
 @enum.unique
 class HDRMode(str, UnknownValuesEnumMixin, enum.Enum):
+    NONE = "none"
     NORMAL = "normal"
     ALWAYS_ON = "superHdr"
 
@@ -586,64 +666,33 @@ class LensType(str, enum.Enum):
     DLSR_17 = "m43"
 
 
-class DoorbellText(ConstrainedStr):
-    max_length = 30
+DoorbellText = Annotated[str, StringConstraints(max_length=30)]
 
+ICRCustomValue = Annotated[int, Field(ge=0, le=11)]
 
-class ICRCustomValue(ConstrainedInt):
-    ge = 0
-    le = 10
+ICRLuxValue = Annotated[int, Field(ge=1, le=30)]
 
+LEDLevel = Annotated[int, Field(ge=0, le=6)]
 
-class ICRLuxValue(ConstrainedInt):
-    ge = 1
-    le = 30
+PercentInt = Annotated[int, Field(ge=0, le=101)]
 
+TwoByteInt = Annotated[int, Field(ge=1, le=256)]
 
-class LEDLevel(ConstrainedInt):
-    ge = 0
-    le = 6
+PercentFloat = Annotated[float, Field(ge=0, le=100)]
 
+WDRLevel = Annotated[int, Field(ge=0, le=4)]
 
-class PercentInt(ConstrainedInt):
-    ge = 0
-    le = 100
+ICRSensitivity = Annotated[int, Field(ge=0, le=4)]
 
+Percent = Annotated[float, Field(ge=0, le=1)]
 
-class TwoByteInt(ConstrainedInt):
-    ge = 1
-    le = 255
-
-
-class PercentFloat(ConstrainedFloat):
-    ge = 0
-    le = 100
-
-
-class WDRLevel(ConstrainedInt):
-    ge = 0
-    le = 3
-
-
-class ICRSensitivity(ConstrainedInt):
-    ge = 0
-    le = 3
-
-
-class Percent(ConstrainedFloat):
-    ge = 0
-    le = 1
-
-
-class RepeatTimes(ConstrainedInt):
-    ge = 1
-    le = 6
+RepeatTimes = Annotated[int, Field(ge=1, le=6)]
 
 
 class PTZPositionDegree(BaseModel):
     pan: float
     tilt: float
-    zoom: int
+    zoom: float
 
 
 class PTZPositionSteps(BaseModel):
@@ -674,15 +723,6 @@ class PTZPreset(BaseModel):
 CoordType = Union[Percent, int, float]
 
 
-# TODO: fix when upgrading to pydantic v2
-class Color(BaseColor):
-    def __eq__(self, o: object) -> bool:
-        if isinstance(o, Color):
-            return self.as_hex() == o.as_hex()
-
-        return super().__eq__(o)
-
-
 class Version(BaseVersion):
     def __str__(self) -> str:
         super_str = super().__str__()
