@@ -103,8 +103,10 @@ class SampleDataGenerator:
     async def async_generate(self, close_session: bool = True) -> None:
         self.log(f"Output folder: {self.output_folder}")
         self.output_folder.mkdir(parents=True, exist_ok=True)
-        websocket = await self.client.get_websocket()
-        websocket.subscribe(self._handle_ws_message)
+        websocket = self.client._get_websocket()
+        websocket.start()
+        self.log("Websocket started...")
+        websocket._subscription = self._handle_ws_message
 
         self.log("Updating devices...")
         await self.client.update()
@@ -124,19 +126,23 @@ class SampleDataGenerator:
             "group": len(bootstrap["groups"]),
             "liveview": len(bootstrap["liveviews"]),
             "viewer": len(bootstrap["viewers"]),
-            "light": len(bootstrap["lights"]),
-            "bridge": len(bootstrap["bridges"]),
-            "sensor": len(bootstrap["sensors"]),
-            "doorlock": len(bootstrap["doorlocks"]),
-            "chime": len(bootstrap["chimes"]),
+            "light": len(bootstrap.get("lights", [])),
+            "bridge": len(bootstrap.get("bridges", [])),
+            "sensor": len(bootstrap.get("sensors", [])),
+            "doorlock": len(bootstrap.get("doorlocks", [])),
+            "chime": len(bootstrap.get("chimes", [])),
+            "aiport": len(bootstrap.get("aiports", [])),
         }
 
+        self.log("Generating event data...")
         motion_event, smart_detection = await self.generate_event_data()
         await self.generate_device_data(motion_event, smart_detection)
+        self.log("Recording websocket events...")
         await self.record_ws_events()
 
         if close_session:
             await self.client.close_session()
+            await self.client.close_public_api_session()
 
         await self.write_json_file("sample_constants", self.constants, anonymize=False)
 
@@ -279,6 +285,7 @@ class SampleDataGenerator:
             self.generate_sensor_data(),
             self.generate_lock_data(),
             self.generate_chime_data(),
+            self.generate_aiport_data(),
             self.generate_bridge_data(),
             self.generate_liveview_data(),
         )
@@ -302,10 +309,18 @@ class SampleDataGenerator:
         await self.write_json_file("sample_camera", deepcopy(obj))
         self.constants["camera_online"] = camera_is_online
 
+        # Check if camera has channels
+        if not obj.get("channels") or len(obj["channels"]) == 0:
+            self.log(
+                "Camera has no channels, skipping snapshot, thumbnail and heatmap generation",
+            )
+            return
+
         if not camera_is_online:
             self.log(
                 "Camera is not online, skipping snapshot, thumbnail and heatmap generation",
             )
+            return
 
         # snapshot
         width = obj["channels"][0]["width"]
@@ -318,6 +333,15 @@ class SampleDataGenerator:
             snapshot = await self.client.get_camera_snapshot(obj["id"], width, height)
             await self.write_image_file(filename, snapshot)
 
+        # public api snapshot
+        pub_filename = "sample_camera_public_api_snapshot"
+        if self.anonymize:
+            self.log(f"Writing {pub_filename}...")
+            placeholder_image(self.output_folder / f"{pub_filename}.png", width, height)
+        else:
+            pub_snapshot = await self.client.get_public_api_camera_snapshot(obj["id"])
+            await self.write_image_file(pub_filename, pub_snapshot)
+
     async def generate_motion_data(
         self,
         motion_event: dict[str, Any] | None,
@@ -355,7 +379,7 @@ class SampleDataGenerator:
         length = int((motion_event["end"] - motion_event["start"]) / 1000)
         if self.anonymize:
             run(
-                split(  # noqa: S603
+                split(
                     BLANK_VIDEO_CMD.format(
                         length=length,
                         filename=self.output_folder / f"{filename}.mp4",
@@ -436,7 +460,12 @@ class SampleDataGenerator:
         await self.write_json_file("sample_sensor", obj)
 
     async def generate_lock_data(self) -> None:
-        objs = await self.client.api_request_list("doorlocks")
+        try:
+            objs = await self.client.api_request_list("doorlocks")
+        except BadRequest:
+            self.log("No doorlock endpoint available. Skipping doorlock endpoints...")
+            return
+
         device_id: str | None = None
         for obj_dict in objs:
             device_id = obj_dict["id"]
@@ -465,6 +494,21 @@ class SampleDataGenerator:
         obj = await self.client.api_request_obj(f"chimes/{device_id}")
         await self.write_json_file("sample_chime", obj)
 
+    async def generate_aiport_data(self) -> None:
+        objs = await self.client.api_request_list("aiports")
+        device_id: str | None = None
+        for obj_dict in objs:
+            device_id = obj_dict["id"]
+            if is_online(obj_dict):
+                break
+
+        if device_id is None:
+            self.log("No aiport found. Skipping aiport endpoints...")
+            return
+
+        obj = await self.client.api_request_obj(f"aiports/{device_id}")
+        await self.write_json_file("sample_aiport", obj)
+
     async def generate_bridge_data(self) -> None:
         objs = await self.client.api_request_list("bridges")
         device_id: str | None = None
